==> walkers-fabric-1.18.2-5.3.3-sources.jar: tocraft/walkers/ability/impl/generic/ShootSnowballAbility.java <==
package tocraft.walkers.ability.impl.generic;

import com.mojang.serialization.MapCodec;
import com.mojang.serialization.codecs.RecordCodecBuilder;
import net.minecraft.class_1308;
import net.minecraft.class_1657;
import net.minecraft.class_1680;
import net.minecraft.class_1792;
import net.minecraft.class_1799;
import net.minecraft.class_1802;
import net.minecraft.class_1937;
import net.minecraft.class_2960;
import net.minecraft.class_3417;
import net.minecraft.class_3419;
import tocraft.walkers.Walkers;
import tocraft.walkers.ability.GenericShapeAbility;

public class ShootSnowballAbility<T extends class_1308> extends GenericShapeAbility<T> {
    public static final class_2960 ID = Walkers.id("shoot_snowball");
    public static final MapCodec<ShootSnowballAbility<?>> CODEC = RecordCodecBuilder.mapCodec((instance) -> instance.stable(new ShootSnowballAbility<>()));

    @Override
    public void onUse(class_1657 player, T shape, class_1937 world) {
        world.method_8465(null, player.method_23317(), player.method_23318(), player.method_23321(), class_3417.field_14873, class_3419.field_15254, 0.5F, 0.4F / (world.field_9229.nextFloat() * 0.4F + 0.8F));

        if (!world.field_9236) {
            for (int i = 0; i < 10; i++) {
                class_1680 snowballEntity = new class_1680(world, player);
                snowballEntity.method_16940(new class_1799(class_1802.field_8543));
                snowballEntity.method_24919(player, player.method_36455() + world.field_9229.nextInt(10) - 5, player.method_36454() + world.field_9229.nextInt(10) - 5, 0.0F, 1.5F, 1.0F);
                world.method_8649(snowballEntity);
            }
        }
    }

    @Override
    public class_1792 getIcon() {
        return class_1802.field_8543;
    }

    @Override
    public int getDefaultCooldown() {
        return 10;
    }

    @Override
    public class_2960 getId() {
        return ID;
    }

    @Override
    public MapCodec<? extends GenericShapeAbility<?>> codec() {
        return CODEC;
    }
}

==> walkers-fabric-1.20.1-5.3.1-sources.jar: tocraft/walkers/ability/impl/generic/ShootSnowballAbility.java <==
package tocraft.walkers.ability.impl.generic;

import com.mojang.serialization.MapCodec;
import com.mojang.serialization.codecs.RecordCodecBuilder;
import net.minecraft.class_1308;
import net.minecraft.class_1657;
import net.minecraft.class_1680;
import net.minecraft.class_1792;
import net.minecraft.class_1799;
import net.minecraft.class_1802;
import net.minecraft.class_1937;
import net.minecraft.class_2960;
import net.minecraft.class_3417;
import net.minecraft.class_3419;
import tocraft.walkers.Walkers;
import tocraft.walkers.ability.GenericShapeAbility;

public class ShootSnowballAbility<T extends class_1308> extends GenericShapeAbility<T> {
    public static final class_2960 ID = Walkers.id("shoot_snowball");
    public static final MapCodec<ShootSnowballAbility<?>> CODEC = RecordCodecBuilder.mapCodec((instance) -> instance.stable(new ShootSnowballAbility<>()));

    @Override
    public void onUse(class_1657 player, T shape, class_1937 world) {
        world.method_43128(null, player.method_23317(), player.method_23318(), player.method_23321(), class_3417.field_14873, class_3419.field_15254, 0.5F, 0.4F / (world.field_9229.method_43057() * 0.4F + 0.8F));

        if (!world.field_9236) {
            for (int i = 0; i < 10; i++) {
                class_1680 snowballEntity = new class_1680(world, player);
                snowballEntity.method_16940(new class_1799(class_1802.field_8543));
                snowballEntity.method_24919(player, player.method_36455() + world.field_9229.method_43048(10) - 5, player.method_36454() + world.field_9229.method_43048(10) - 5, 0.0F, 1.5F, 1.0F);
                world.method_8649(snowballEntity);
            }
        }
    }

    @Override
    public class_1792 getIcon() {
        return class_1802.field_8543;
    }

    @Override
    public int getDefaultCooldown() {
        return 10;
    }

    @Override
    public class_2960 getId() {
        return ID;
    }

    @Override
    public MapCodec<? extends GenericShapeAbility<?>> codec() {
        return CODEC;
    }
}
